//  Copyright © 2012 Stephen Smally
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.
//      

using PackageKit;
using SQLHeavy;

namespace Lsc.Backend { 
    public class AppsManager : Object {
        // Signals
        public signal void app_added (LscApp app);
        public signal void category_added (LscCategory cat);
        public signal void loading_started (LoadingType load, string text);
        public signal void loading_finished (LoadingType load);
        public signal void details_received (LscApp app, Package pkg, Details details);
        
        // PackageKit stuffs
        private Client client;
        private Control control;
        private Task task;
        private Cancellable transaction;
        
        // SQLHeavy stuffs
        private Database database;
        
        public void remove_packages (string[] ids, bool deps, bool auto) {
			task.remove_packages_async (ids,
			                            deps,
			                            auto,
			                            null,
			                            (p) => {
											if (p.percentage != -1) {
												stdout.printf ("%d%\n", p.percentage);
											}
										},
			                            () => {
										    stdout.printf ("Finished to remove\n");
										});
		}
        
        public void install_packages (string[] ids) {
			task.install_packages_async (ids,
			                             null,
			                             (p) => {
											 if (p.percentage != -1) {
										         stdout.printf ("%d%\n", p.percentage);
											 }
										 },
			                             () => {
											 stdout.printf ("Finished to install\n");
										 });
		}
        
        public void get_pkgs (string category) {
            try {
				Query get_query = database.prepare ("SELECT * FROM '%s';".printf (category));
				QueryResult result = get_query.execute ();
				
				for (int record = 1; ! result.finished; record++, result.next ()) {
					app_added (new LscApp (
					           result.fetch_string (0),
					           result.fetch_string (1),
					           result.fetch_string (2),
					           result.fetch_string (3)));
				}
			} catch (GLib.Error e) {
				GLib.error ("Error retrieving packages: %s\n", e.message);
			}
				
        }
        
        public void get_details_2 (LscApp pkg, Package p) {
			client.get_details_async({p.get_id(), null},
            null,
            () => {
				while (Gtk.events_pending()) {
                    Gtk.main_iteration();
				}
			},
            (obj, resu) => {
				Results res = client.generic_finish (resu);
				details_received (pkg, p, res.get_details_array()[0]);
				loading_finished (LoadingType.DETAILS);
			}
            );
		}
        
        public void get_details (LscApp pkg) {
			loading_started (LoadingType.DETAILS, "Loading infos...");
			client.resolve_async(0,
            {pkg.id, null},
            null,
            () => {
				while (Gtk.events_pending()) {
                    Gtk.main_iteration();
				}
			},
            (obj, resu) => {
				Results res = client.generic_finish (resu);
				get_details_2 (pkg, res.get_package_array()[0]);
			});
        }
        
        public int count_category (string category) {
			Query q = database.prepare ("SELECT COUNT (*) FROM '%s';".printf (category));
            QueryResult r = q.execute();
            return r.fetch_int (0);
		}
        
        public void get_categories () {
			loading_started(LoadingType.CATEGORIES, "Loading categories...");
            try {
				Query cat_query = database.prepare ("SELECT * FROM 'DIRECTORIES';");
				QueryResult result = cat_query.execute ();
				
				for (int record = 1; ! result.finished; record++, result.next ()) {
					category_added (new LscCategory (result.fetch_string (0),
								  result.fetch_string (1),
								  result.fetch_string (2),
								  result.fetch_string (3),
								  count_category (result.fetch_string (0))));
				}
			} catch (GLib.Error e) {
				GLib.error ("Error retrieving packages: %s\n", e.message);
			}
            
            loading_finished(LoadingType.CATEGORIES);
        }
        
        public AppsManager () {
            client = new Client();
            control = new Control();
            task = new Task();
            transaction = new Cancellable();
            
            try {
	            database = new Database ("/var/cache/lsc-vala.db", FileMode.READ);
			} catch (GLib.Error e) {
				GLib.error ("Error opening the database: %s\nPlease run 'lsc-db-build -d /var/cache/lsc-packages.db' as root", e.message);
			}
			
			try_get();
        }
    }
}
